==> spring-cloud-gateway-core-2.0.1.RELEASE-sources.jar: org/springframework/cloud/gateway/support/ConfigurationUtils.java <==
/*
 * Copyright 2013-2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.springframework.cloud.gateway.support;

import java.util.Map;

import org.springframework.aop.framework.Advised;
import org.springframework.aop.support.AopUtils;
import org.springframework.boot.context.properties.bind.Bindable;
import org.springframework.boot.context.properties.bind.Binder;
import org.springframework.boot.context.properties.source.MapConfigurationPropertySource;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Validator;

public abstract class ConfigurationUtils {

	public static void bind(Object o, Map<String, Object> properties, String configurationPropertyName, String bindingName, Validator validator) {
		Object toBind = getTargetObject(o);

		new Binder(new MapConfigurationPropertySource(properties))
                .bind(configurationPropertyName, Bindable.ofInstance(toBind));

		if (validator != null) {
			BindingResult errors = new BeanPropertyBindingResult(toBind, bindingName);
			validator.validate(toBind, errors);
			if (errors.hasErrors()) {
				throw new RuntimeException(new BindException(errors));
			}
		}
	}

	@SuppressWarnings("unchecked")
	public static <T> T getTargetObject(Object candidate) {
		try {
			if (AopUtils.isAopProxy(candidate) && (candidate instanceof Advised)) {
				return (T) ((Advised) candidate).getTargetSource().getTarget();
			}
		}
		catch (Exception ex) {
			throw new IllegalStateException("Failed to unwrap proxied object", ex);
		}
		return (T) candidate;
	}
}

==> spring-cloud-gateway-core-2.1.0.RELEASE-sources.jar: org/springframework/cloud/gateway/support/ConfigurationUtils.java <==
/*
 * Copyright 2013-2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.springframework.cloud.gateway.support;

import java.util.Collections;
import java.util.Map;

import org.springframework.aop.framework.Advised;
import org.springframework.aop.support.AopUtils;
import org.springframework.boot.context.properties.bind.Bindable;
import org.springframework.boot.context.properties.bind.Binder;
import org.springframework.boot.context.properties.source.MapConfigurationPropertySource;
import org.springframework.core.convert.ConversionService;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Validator;

public abstract class ConfigurationUtils {

	public static void bind(Object o, Map<String, Object> properties, String configurationPropertyName, String bindingName, Validator validator) {
		bind(o, properties, configurationPropertyName, bindingName, validator, null);
	}

	public static void bind(Object o, Map<String, Object> properties, String configurationPropertyName, String bindingName,
							Validator validator, ConversionService conversionService) {
		Object toBind = getTargetObject(o);

		new Binder(Collections.singletonList(new MapConfigurationPropertySource(properties)), null, conversionService)
                .bind(configurationPropertyName, Bindable.ofInstance(toBind));

		if (validator != null) {
			BindingResult errors = new BeanPropertyBindingResult(toBind, bindingName);
			validator.validate(toBind, errors);
			if (errors.hasErrors()) {
				throw new RuntimeException(new BindException(errors));
			}
		}
	}

	@SuppressWarnings("unchecked")
	public static <T> T getTargetObject(Object candidate) {
		try {
			if (AopUtils.isAopProxy(candidate) && (candidate instanceof Advised)) {
				return (T) ((Advised) candidate).getTargetSource().getTarget();
			}
		}
		catch (Exception ex) {
			throw new IllegalStateException("Failed to unwrap proxied object", ex);
		}
		return (T) candidate;
	}
}
